@@ -8,12 +8,27 @@
 #' @param zname an optional character specifying the name of the treatment variable in \code{data}. Unless \code{ps.formula} is specified, \code{zname} is required.
 #' @param delta trimming threshold for estimated (generalized) propensity scores. Should be no larger than 1 / number of treatment groups. Default is 0, corresponding to no trimming.
 #' @param optimal an logical argument indicating if optimal trimming should be used. Default is \code{FALSE}.
+#' @param out.estimate an optional matrix or data frame containing estimated potential outcomes
+#' for each observation. Typically, this is an N by J matrix, where N is the number of observations
+#' and J is the total number of treatment levels. Preferably, the column name of this matrix should
+#' match the name of treatment level, if column name is missing or there is a mismatch,
+#' the column names would be assigned according to alphabatic order of the treatment levels, with a
+#' similar mechanism as in \code{ps.estimate}.
+#' @param method a character to specify the method for propensity model. \code{"glm"} is default, and \code{"gbm"} and \code{"SuperLearner"} are also allowed.
+#' @param ps.control a list to specify addtional optionals when \code{method} is set to \code{"gbm"} or \code{"SuperLearner"}.
 #'
 #' @details A typical form for \code{ps.formula} is \code{treatment ~ terms} where \code{treatment} is the treatment
 #' variable (identical to the variable name used to specify \code{zname}) and \code{terms} is a series of terms
-#' which specifies a linear predictor for \code{treatment}. \code{ps.formula} specifies generalized linear
-#' model for estimating the propensity scores, when \code{ps.estimate} is \code{NULL}. See \code{glm} for
-#' more details on generalized linear models.
+#' which specifies a linear predictor for \code{treatment}. \code{ps.formula} specifies a
+#' model for estimating the propensity scores, when \code{ps.estimate} is \code{NULL}.
+#'  \code{"glm"} is the default method for propensity score estimation. Logistic regression will be used for binary outcomes,
+#' and multinomial logistic regression will be used for outcomes with more than two categories. The alternative method option of \code{"gbm"} serves as an API to call the \code{gbm()} function from the
+#' \code{gbm} package. Additional argument in the \code{gbm()} function can be supplied through the \code{ps.control=list()} argument in \code{SumStat()}. Please refer to the user manual of the gbm package for all the
+#' allowed arguments. Currently, models for binary or multinomial treatment will be automatically chosen based on the number of treatment categories.
+#' \code{"SuperLearner"} is also allowed in the \code{method} argument to pass the propensity score estimation to the \code{SuperLearner()} function in \code{SuperLearner} package.
+#' Currently, the SuperLearner method only support binary treatment with the default method set to \code{"SL.glm"}. The estimation approach is fixed to \code{"method.NNLS"}.
+#' Prediction algorithm and other tuning parameters can also be passed through \code{ps.control=list()}. Please refer to the user manual of the \code{SuperLearner} package for all the allowed specifications.
+#'
 #'
 #' When comparing two treatments, \code{ps.estimate} can either be a vector or a two-column matrix of estimated
 #' propensity scores. If a vector is supplied, it is assumed to be the propensity scores to receive the treatment, and
@@ -21,6 +36,9 @@
 #' When comparing multiple (J>=3) treatments, \code{ps.estimate} needs to be specified as an N by J matrix,
 #' where N indicates the number of observations, and J indicates the total number of treatments.
 #' This matrix specifies the estimated generalized propensity scores to receive each of the J treatments.
+#' The same mechanism applies to \code{out.estimate}, except that the input for \code{out.estimate}
+#' must be an N by J matrix, where each row corresponds to the estimated potential outcomes (corresponding to each treatment)
+#' for each observation.
 #'
 #' With binary treatments, \code{delta} defines the symmetric propensity score trimming rule following Crump et al. (2009).
 #' With multiple treatments, \code{delta} defines the symmetric multinomial trimming rule introduced in Yoshida et al. (2019).
@@ -44,6 +62,8 @@
 #' \item{\code{ delta}}{ an optional output of trimming threshold for symmetric trimming. }
 #'
 #' \item{\code{ lambda}}{ an optional output trimming threshold for optimal trimming with multiple treatment groups. }
+#'
+#' \item{\code{ out.estimate}}{ a data frame of estimated potential outcomes after trimming. }
 #' }
 #'
 #'
@@ -83,26 +103,20 @@
 
 
 
-PStrim<-function(data,ps.formula=NULL,zname=NULL,ps.estimate=NULL,delta=0,optimal=FALSE){
+PStrim<-function(data,ps.formula=NULL,zname=NULL,ps.estimate=NULL,delta=0,optimal=FALSE,out.estimate=NULL,method='glm',ps.control=list()){
 
   #extract zname
-  if (is.null(ps.formula)){
-    if (is.null(zname)) stop("treatment variable zname required","\n")
+  if(!is.null(ps.formula)){
+    ps.formula<-as.formula(ps.formula)
+    zname<-all.vars(ps.formula)[1]
   }
 
-  if(typeof(ps.formula)!="character"){
-    ps.formula<-Reduce(paste0,deparse(ps.formula))
-  }
-  ps.formula<-gsub(" ","",ps.formula)
-
-  if(is.null(zname)){
-    zname<-unlist(strsplit(ps.formula,'~'))[[1]][1]
-  }
 
   datatmp<-data
   z<-as.factor(unlist(data[zname]))
   datatmp[zname]<-z
-  ncate<-length(unique(z))
+  ncate<-length(levels(z))
+  n<-length(z)
 
   if(!optimal){
     if(is.null(ps.estimate)){
@@ -110,16 +124,11 @@ PStrim<-function(data,ps.formula=NULL,zname=NULL,ps.estimate=NULL,delta=0,optima
       if(1/ncate<=delta){
         warning('invalid trimming, return original data')
       }else{
-        if(ncate==2){
-          fittrim <- glm(ps.formula, family = binomial(link = "logit"),data=datatmp)
-          propensity<-cbind(1-fittrim$fitted.values,fittrim$fitted.values)
-        }else{
-          fittrim<- nnet::multinom(formula = ps.formula, data=datatmp,maxit = 500, Hess = TRUE, trace = FALSE)
-          propensity<-fittrim$fitted.values
-        }
+        propensity<-do.call(PSmethod,c(list(ps.formula = ps.formula, method=method, data=datatmp,ncate=ncate),ps.control))$e.h
         psidx<-apply(as.matrix(propensity),1,function(x) min(x)>delta)
         if(length(unique(z[psidx]))==ncate){
           data<-data[psidx,]
+          out.estimate<-out.estimate[psidx,]
         }else{
           warning('One or more groups removed after trimming, reset your delta, trimming not applied')
         }
@@ -128,16 +137,16 @@ PStrim<-function(data,ps.formula=NULL,zname=NULL,ps.estimate=NULL,delta=0,optima
       if(1/ncate<=delta){
         warning('invalid trimming, return original data')
       }else{
-        if(is.vector(ps.estimate)){
-          ps.estimate<-cbind(1-ps.estimate,ps.estimate)
-        }else if(ncol(ps.estimate)==1){
+        if(length(ps.estimate)==n){
           ps.estimate<-cbind(1-ps.estimate,ps.estimate)
         }
+
         psidx<-apply(as.matrix(ps.estimate),1,function(x) min(x)>delta)
 
         if(length(unique(z[psidx]))==ncate){
           data<-data[psidx,]
           ps.estimate<-ps.estimate[psidx,]
+          out.estimate<-out.estimate[psidx,]
         }
       }
     }
@@ -147,12 +156,9 @@ PStrim<-function(data,ps.formula=NULL,zname=NULL,ps.estimate=NULL,delta=0,optima
     if (delta>0) warning('delta is ignored when optimal trimming is used')
     if (ncate==2){
       if(is.null(ps.estimate)){
-        fittrim <- glm(ps.formula, family = binomial(link = "logit"),data=datatmp)
-        propensity<-cbind(1-fittrim$fitted.values,fittrim$fitted.values)
+        propensity<-do.call(PSmethod,c(list(ps.formula = ps.formula, method=method, data=datatmp,ncate=ncate),ps.control))$e.h
       }else{
-        if(is.vector(ps.estimate)){
-          ps.estimate<-cbind(1-ps.estimate,ps.estimate)
-        }else if(ncol(ps.estimate)==1){
+        if(length(ps.estimate)==n){
           ps.estimate<-cbind(1-ps.estimate,ps.estimate)
         }
         propensity<-ps.estimate
@@ -175,8 +181,7 @@ PStrim<-function(data,ps.formula=NULL,zname=NULL,ps.estimate=NULL,delta=0,optima
       }
     }else{
       if(is.null(ps.estimate)){
-        fittrim<- nnet::multinom(formula = ps.formula, data=datatmp,maxit = 500, Hess = TRUE, trace = FALSE)
-        propensity<-fittrim$fitted.values
+        propensity<-do.call(PSmethod,c(list(ps.formula = ps.formula, method=method, data=datatmp,ncate=ncate),ps.control))$e.h
       }else{
         propensity<-ps.estimate
       }
@@ -198,9 +203,9 @@ PStrim<-function(data,ps.formula=NULL,zname=NULL,ps.estimate=NULL,delta=0,optima
     }
 
     if (ncate==2){
-      delta=tolg
+      delta<-tolg
       psidx<-apply(as.matrix(propensity),1,function(x) min(x)>delta)
-    }else{ psidx=keep }
+    }else{ psidx<-keep }
 
     if(length(unique(z[psidx]))==ncate){
       data<-data[psidx,]
@@ -208,9 +213,9 @@ PStrim<-function(data,ps.formula=NULL,zname=NULL,ps.estimate=NULL,delta=0,optima
       warning('One or more groups removed after trimming, reset your delta, trimming not applied')
     }
 
-    if (!is.null(ps.estimate)){
-      ps.estimate<-ps.estimate[psidx,]
-    }
+
+    ps.estimate<-ps.estimate[psidx,]
+    out.estimate<-out.estimate[psidx,]
   }
 
   trimmed<-table(datatmp[zname])-table(data[zname])
@@ -218,12 +223,15 @@ PStrim<-function(data,ps.formula=NULL,zname=NULL,ps.estimate=NULL,delta=0,optima
   trim_sum<-rbind(trimmed,remained)
 
   if (ncate>2 & optimal){
-    return(list(data=data,trim_sum=trim_sum,ps.estimate=ps.estimate,delta=NULL,lambda=lambda))
+    out<-list(data=data,trim_sum=trim_sum,ps.estimate=ps.estimate,delta=NULL,lambda=lambda, out.estimate=out.estimate)
   }
   else{
-    return(list(data=data,trim_sum=trim_sum,ps.estimate=ps.estimate,delta=delta,lambda=NULL))
+    out<-list(data=data,trim_sum=trim_sum,ps.estimate=ps.estimate,delta=delta,lambda=NULL, out.estimate=out.estimate)
   }
 
+  class(out)<-'PStrim'
+  out
+
 }
 
 
